//
//  SpaceshipScene.m
//  Space Simulator
//
//  Created by Simon Rummert on 14.08.13.
//  Copyright (c) 2013 Simon Rummert. All rights reserved.
//

#import "SpaceshipScene.h"

@interface SpaceshipScene ()
@property BOOL contentCreated;

@end

@implementation SpaceshipScene

static const uint32_t shipCategory        =  0x1 << 1;
static const uint32_t starCategory        =  0x1 << 2;

- (void)didMoveToView:(SKView *)view
{
    if (!self.contentCreated)
    {
        [self createSceneContents];
        self.contentCreated = YES;
    }
}

- (void)createSceneContents
{
    self.backgroundColor = [SKColor blackColor];
    self.scaleMode = SKSceneScaleModeAspectFit;
    SKSpriteNode *spaceship = [self newSpaceship];
    spaceship.position = CGPointMake(CGRectGetMidX(self.frame),                              CGRectGetMidY(self.frame)-150);
    [self addChild:spaceship];
    SKAction *makeStars = [SKAction sequence: @[
                                                [SKAction performSelector:@selector(addStar) onTarget:self],
                                                [SKAction waitForDuration:0.10 withRange:0.15]
                                                ]];
    [self runAction: [SKAction repeatActionForever:makeStars]];
    self.physicsWorld.contactDelegate = self;
}

- (SKSpriteNode *)newSpaceship
{
    SKSpriteNode *spaceship = [SKSpriteNode spriteNodeWithImageNamed:@"Spaceship.png"];
    
    spaceship.xScale = 0.25;
    spaceship.yScale = 0.25;
    
    SKAction *hover = [SKAction sequence:@[
                                           [SKAction waitForDuration:1.0],
                                           [SKAction moveByX:100 y:50.0 duration:1.0],
                                           [SKAction waitForDuration:1.0],
                                           [SKAction moveByX:-100.0 y:-50 duration:1.0]]];
    [spaceship runAction: [SKAction repeatActionForever:hover]];
    
#warning Exercise 2: instantiate the rocket trail
    
    
#warning Exercise 2: add rocket trail to spaceship node
    
    spaceship.physicsBody = [SKPhysicsBody bodyWithCircleOfRadius:0.4*spaceship.size.width];
    spaceship.physicsBody.dynamic = NO;
    spaceship.physicsBody.categoryBitMask = shipCategory;
    spaceship.physicsBody.collisionBitMask = shipCategory | starCategory;
    spaceship.physicsBody.contactTestBitMask = shipCategory | starCategory;
    
    return spaceship;
}



static inline CGFloat skRandf() {
    return rand() / (CGFloat) RAND_MAX;
}

static inline CGFloat skRand(CGFloat low, CGFloat high) {
    return skRandf() * (high - low) + low;
}

- (void)addStar
{
    SKSpriteNode *star = [[SKSpriteNode alloc] initWithImageNamed:@"Star.png"];
    star.xScale = 0.25;
    star.yScale = 0.25;
    star.position = CGPointMake(skRand(0, self.size.width), self.size.height);
    star.name = @"star";
    star.physicsBody = [SKPhysicsBody bodyWithCircleOfRadius:0.5*star.size.width];
    
    /* If you want rebounding stars instead of explosions.       *
     * You need to remove collision detection in order to do so  *
     *                                                           *
     //star.physicsBody.usesPreciseCollisionDetection = YES;     */
    
    
    [self addChild:star];
}

-(void)didSimulatePhysics
{
    [self enumerateChildNodesWithName:@"star" usingBlock:^(SKNode *node, BOOL *stop) {
        if (node.position.y < 0)
            [node removeFromParent];
        
    }];
}


//Called when two bodies first contact each other.
-(void)didBeginContact:(SKPhysicsContact *)contact
{
#warning Exercise 1: insantiate new explosion on contact
    
    
#warning Exercise 1: set the position of contact
    
    
    [contact.bodyB.node removeFromParent];
    
}

-(SKEmitterNode *)newExplosion
{
    
#warning Exercise 1: instantiate explosion emitter
    
    
#warning Exercise 1: set the properties for your particle system
    /*
    [explosion setParticleTexture:<#(SKTexture *)#>];
    [explosion setParticleColor:<#(UIColor *)#>];
    [explosion setNumParticlesToEmit:<#(NSUInteger)#>];
    [explosion setParticleBirthRate:<#(CGFloat)#>];
    [explosion setParticleLifetime:<#(CGFloat)#>];
    [explosion setEmissionAngleRange:<#(CGFloat)#>];
    [explosion setParticleSpeed:<#(CGFloat)#>];
    [explosion setParticleSpeedRange:<#(CGFloat)#>];
    [explosion setXAcceleration:<#(CGFloat)#>];
    [explosion setYAcceleration:<#(CGFloat)#>];
    [explosion setParticleAlpha:<#(CGFloat)#>];
    [explosion setParticleAlphaRange:<#(CGFloat)#>];
    [explosion setParticleAlphaSpeed:<#(CGFloat)#>];
    [explosion setParticleScale:<#(CGFloat)#>];
    [explosion setParticleScaleRange:<#(CGFloat)#>];
    [explosion setParticleScaleSpeed:<#(CGFloat)#>];
    [explosion setParticleRotation:<#(CGFloat)#>];
    [explosion setParticleRotationRange:<#(CGFloat)#>];
    [explosion setParticleRotationSpeed:<#(CGFloat)#>];
    
    
    //Blending properties (you don't need to change these)
    [explosion setParticleColorBlendFactor:1];
    [explosion setParticleColorBlendFactorRange:0];
    [explosion setParticleColorBlendFactorSpeed:0];
    [explosion setParticleBlendMode:SKBlendModeAdd];
    */
    
#warning Exercise 1: add the explosion node to parent node
    
    
    return nil;
     
}




- (SKEmitterNode *) newFireEmitter
{
    
#warning Exercise 2: create path to your particle file
    
    
#warning Exercise 2: instantiate EmitterNode with your filepath and return it
    return nil;
}


@end